#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <X11/X.h>
#include <X11/xpm.h>
#include "wireless.h"
#include <net/ethernet.h>
#include <linux/if.h>
#include <linux/socket.h>
#include <linux/wireless.h>

/*
 * This is my quick and dirty way of accessing the Linux Wireless Extensions
 * from the /proc/net/wireless interface.
 * Open the file, and get the Interface name, link, level, and noise. 
 * - Jess
 */

int get_max_qual(char ifname[255])
{
    struct iwreq wrq;
    struct iw_range range;
    int skfd, ret;


    skfd = socket(AF_INET, SOCK_DGRAM, 0);

    memset(&range, 0, sizeof(struct iw_range));

    wrq.u.data.pointer = (caddr_t) & range;
    wrq.u.data.length = sizeof(struct iw_range);
    wrq.u.data.flags = 0;
    strncpy(wrq.ifr_name, ifname, IFNAMSIZ);
    ret = ioctl(skfd, SIOCGIWRANGE, &wrq);

    if (ret == -1) {
	fprintf(stderr,
		"ERROR: Problem communicating with Linux Wireless Extensions.\n"
		"Please make sure your wireless driver/module is compiled or loaded\n"
		"into the kernel and configured properly. If it is, and you still get\n"
		"this error, then...\n"
		"Please send a bug report to jess@digitalssg.net\n");
	exit(1);
    }
    /* Version verification - for new versions */
    if (range.we_version_compiled != WIRELESS_EXT) {
	fprintf(stderr,
		"Warning: Driver for device %s has been compiled with version %d\n",
		ifname, range.we_version_compiled);
	fprintf(stderr,
		"of Wireless Extension, while this program is using version %d.\n",
		WIRELESS_EXT);
	fprintf(stderr, "Some things may be broken...\n\n");
    }
    close(skfd);
    return range.max_qual.qual;
}

struct wifi wifi_info(void)
{
    FILE *fp;
    char buff[255], level[255];
    struct wifi wfi;

    if (fp = fopen("/proc/net/wireless", "r")) {
	fgets(buff, sizeof(buff), fp);
	fgets(buff, sizeof(buff), fp);
	fgets(buff, sizeof(buff), fp);
	sscanf(buff, "%s %*s %s %*s %*s", wfi.ifname, level);

	wfi.link = atoi(level);
	wfi.max_qual = get_max_qual(wfi.ifname);
//          fprintf(stdout, "Ifname: %s, Link: %d/%-f\n", wfi.ifname, wfi.link, wfi.max_qual);
	fclose(fp);
    } else {
	fprintf(stderr,
		"ERROR: Cannot Open Linux Wireless Extensions!\n\b");
	exit(1);
    }
    return wfi;
}
